package javax.swing.tree;

import java.awt.Graphics;
import java.awt.Dimension;
import javax.swing.plaf.FontUIResource;
import javax.swing.border.Border;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.Vector;
import java.io.ObjectOutputStream;
import javax.swing.JTextField;
import java.awt.Rectangle;
import javax.swing.SwingUtilities;
import java.awt.event.ActionEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.DefaultCellEditor;
import javax.swing.event.CellEditorListener;
import java.awt.event.MouseEvent;
import java.util.EventObject;
import javax.swing.UIManager;
import java.awt.Font;
import javax.swing.Icon;
import java.awt.Color;
import javax.swing.Timer;
import javax.swing.JTree;
import java.awt.Component;
import java.awt.Container;
import javax.swing.event.TreeSelectionListener;
import java.awt.event.ActionListener;

public class DefaultTreeCellEditor implements ActionListener, TreeCellEditor, TreeSelectionListener
{
    protected TreeCellEditor realEditor;
    protected DefaultTreeCellRenderer renderer;
    protected Container editingContainer;
    protected transient Component editingComponent;
    protected boolean canEdit;
    protected transient int offset;
    protected transient JTree tree;
    protected transient TreePath lastPath;
    protected transient Timer timer;
    protected transient int lastRow;
    protected Color borderSelectionColor;
    protected transient Icon editingIcon;
    protected Font font;
    
    public DefaultTreeCellEditor(JTree tree, DefaultTreeCellRenderer defaultTreeCellRenderer) {
        this(tree, defaultTreeCellRenderer, null);
    }
    
    public DefaultTreeCellEditor(JTree tree, DefaultTreeCellRenderer renderer, TreeCellEditor realEditor) {
        this.renderer = renderer;
        this.realEditor = realEditor;
        if (this.realEditor == null) {
            this.realEditor = this.createTreeCellEditor();
        }
        this.editingContainer = this.createContainer();
        this.setTree(tree);
        this.setBorderSelectionColor(UIManager.getColor("Tree.editorBorderSelectionColor"));
    }
    
    public void setBorderSelectionColor(Color borderSelectionColor) {
        this.borderSelectionColor = borderSelectionColor;
    }
    
    public Color getBorderSelectionColor() {
        return this.borderSelectionColor;
    }
    
    public void setFont(Font font) {
        this.font = font;
    }
    
    public Font getFont() {
        return this.font;
    }
    
    @Override
    public Component getTreeCellEditorComponent(JTree tree, Object o, boolean b, boolean b2, boolean b3, int n) {
        this.setTree(tree);
        this.determineOffset(tree, o, b, b2, b3, this.lastRow = n);
        if (this.editingComponent != null) {
            this.editingContainer.remove(this.editingComponent);
        }
        this.editingComponent = this.realEditor.getTreeCellEditorComponent(tree, o, b, b2, b3, n);
        TreePath pathForRow = tree.getPathForRow(n);
        this.canEdit = (this.lastPath != null && pathForRow != null && this.lastPath.equals(pathForRow));
        Font font = this.getFont();
        if (font == null) {
            if (this.renderer != null) {
                font = this.renderer.getFont();
            }
            if (font == null) {
                font = tree.getFont();
            }
        }
        this.editingContainer.setFont(font);
        this.prepareForEditing();
        return this.editingContainer;
    }
    
    @Override
    public Object getCellEditorValue() {
        return this.realEditor.getCellEditorValue();
    }
    
    @Override
    public boolean isCellEditable(EventObject eventObject) {
        boolean n = false;
        boolean n2 = false;
        if (eventObject != null && eventObject.getSource() instanceof JTree) {
            this.setTree((JTree)eventObject.getSource());
            if ((eventObject instanceof MouseEvent)) {
                TreePath pathForLocation = this.tree.getPathForLocation(((MouseEvent)eventObject).getX(), ((MouseEvent)eventObject).getY());
                n2 = (this.lastPath != null && pathForLocation != null && this.lastPath.equals(pathForLocation));
                if (pathForLocation != null) {
                    this.lastRow = this.tree.getRowForPath(pathForLocation);
                    Object lastPathComponent = pathForLocation.getLastPathComponent();
                    this.determineOffset(this.tree, lastPathComponent, this.tree.isRowSelected(this.lastRow), this.tree.isExpanded(pathForLocation), this.tree.getModel().isLeaf(lastPathComponent), this.lastRow);
                }
            }
        }
        if (!this.realEditor.isCellEditable(eventObject)) {
            return false;
        }
        if (this.canEditImmediately(eventObject)) {
            n = true;
        }
        else if (n2 && this.shouldStartEditingTimer(eventObject)) {
            this.startEditingTimer();
        }
        else if (this.timer != null && this.timer.isRunning()) {
            this.timer.stop();
        }
        if (!(n == 0)) {
            this.prepareForEditing();
        }
        return n;
    }
    
    @Override
    public boolean shouldSelectCell(EventObject eventObject) {
        return this.realEditor.shouldSelectCell(eventObject);
    }
    
    @Override
    public boolean stopCellEditing() {
        if (this.realEditor.stopCellEditing()) {
            this.cleanupAfterEditing();
            return true;
        }
        return false;
    }
    
    @Override
    public void cancelCellEditing() {
        this.realEditor.cancelCellEditing();
        this.cleanupAfterEditing();
    }
    
    @Override
    public void addCellEditorListener(CellEditorListener cellEditorListener) {
        this.realEditor.addCellEditorListener(cellEditorListener);
    }
    
    @Override
    public void removeCellEditorListener(CellEditorListener cellEditorListener) {
        this.realEditor.removeCellEditorListener(cellEditorListener);
    }
    
    public CellEditorListener[] getCellEditorListeners() {
        return ((DefaultCellEditor)this.realEditor).getCellEditorListeners();
    }
    
    @Override
    public void valueChanged(TreeSelectionEvent treeSelectionEvent) {
        if (this.tree != null) {
            if (this.tree.getSelectionCount() == 1) {
                this.lastPath = this.tree.getSelectionPath();
            }
            else {
                this.lastPath = null;
            }
        }
        if (this.timer != null) {
            this.timer.stop();
        }
    }
    
    @Override
    public void actionPerformed(ActionEvent actionEvent) {
        if (this.tree != null && this.lastPath != null) {
            this.tree.startEditingAtPath(this.lastPath);
        }
    }
    
    protected void setTree(JTree tree) {
        if (this.tree != tree) {
            if (this.tree != null) {
                this.tree.removeTreeSelectionListener(this);
            }
            this.tree = tree;
            if (this.tree != null) {
                this.tree.addTreeSelectionListener(this);
            }
            if (this.timer != null) {
                this.timer.stop();
            }
        }
    }
    
    protected boolean shouldStartEditingTimer(EventObject eventObject) {
        if (eventObject instanceof MouseEvent && SwingUtilities.isLeftMouseButton((MouseEvent)eventObject)) {
            MouseEvent mouseEvent = (MouseEvent)eventObject;
            return mouseEvent.getClickCount() == 1 && this.inHitRegion(mouseEvent.getX(), mouseEvent.getY());
        }
        return false;
    }
    
    protected void startEditingTimer() {
        if (this.timer == null) {
            (this.timer = new Timer(1200, this)).setRepeats(false);
        }
        this.timer.start();
    }
    
    protected boolean canEditImmediately(EventObject eventObject) {
        if (eventObject instanceof MouseEvent && SwingUtilities.isLeftMouseButton((MouseEvent)eventObject)) {
            MouseEvent mouseEvent = (MouseEvent)eventObject;
            return mouseEvent.getClickCount() > 2 && this.inHitRegion(mouseEvent.getX(), mouseEvent.getY());
        }
        return eventObject == null;
    }
    
    protected boolean inHitRegion(int n, int n2) {
        if (this.lastRow != -1 && this.tree != null) {
            Rectangle rowBounds = this.tree.getRowBounds(this.lastRow);
            if (this.tree.getComponentOrientation().isLeftToRight()) {
                if (rowBounds != null && n <= rowBounds.x + this.offset && this.offset < rowBounds.width - 5) {
                    return false;
                }
            }
            else if (rowBounds != null && (n >= rowBounds.x + rowBounds.width - this.offset + 5 || n <= rowBounds.x + 5) && this.offset < rowBounds.width - 5) {
                return false;
            }
        }
        return true;
    }
    
    protected void determineOffset(JTree tree, Object o, boolean b, boolean b2, boolean b3, int n) {
        if (this.renderer != null) {
            if (b3) {
                this.editingIcon = this.renderer.getLeafIcon();
            }
            else if (b2) {
                this.editingIcon = this.renderer.getOpenIcon();
            }
            else {
                this.editingIcon = this.renderer.getClosedIcon();
            }
            if (this.editingIcon != null) {
                this.offset = this.renderer.getIconTextGap() + this.editingIcon.getIconWidth();
            }
            else {
                this.offset = this.renderer.getIconTextGap();
            }
        }
        else {
            this.editingIcon = null;
            this.offset = 0;
        }
    }
    
    protected void prepareForEditing() {
        if (this.editingComponent != null) {
            this.editingContainer.add(this.editingComponent);
        }
    }
    
    protected Container createContainer() {
        return new DefaultTreeCellEditor.EditorContainer();
    }
    
    protected TreeCellEditor createTreeCellEditor() {
        DefaultCellEditor defaultCellEditor = new DefaultCellEditor(new DefaultTreeCellEditor.DefaultTextField(UIManager.getBorder("Tree.editorBorder"))) {
            @Override
            public boolean shouldSelectCell(EventObject anEvent) {
                return super.shouldSelectCell(anEvent);
            }
        };
        defaultCellEditor.setClickCountToStart(1);
        return defaultCellEditor;
    }
    
    private void cleanupAfterEditing() {
        if (this.editingComponent != null) {
            this.editingContainer.remove(this.editingComponent);
        }
        this.editingComponent = null;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Vector obj = new Vector();
        objectOutputStream.defaultWriteObject();
        if (this.realEditor != null && this.realEditor instanceof Serializable) {
            obj.addElement("realEditor");
            obj.addElement(this.realEditor);
        }
        objectOutputStream.writeObject(obj);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        Vector<Object> vector = (Vector)objectInputStream.readObject();
        int index = 0;
        if (index < vector.size() && vector.elementAt(index).equals("realEditor")) {
            this.realEditor = (TreeCellEditor)vector.elementAt(++index);
            ++index;
        }
    }
    
    public class DefaultTextField extends JTextField
    {
        protected Border border;
        
        public DefaultTextField(Border border) {
            this.setBorder(border);
        }
        
        @Override
        public void setBorder(Border border) {
            super.setBorder(border);
            this.border = border;
        }
        
        @Override
        public Border getBorder() {
            return this.border;
        }
        
        @Override
        public Font getFont() {
            Font font = super.getFont();
            if ((font instanceof FontUIResource)) {
                Container parent = this.getParent();
                if (parent != null && parent.getFont() != null) {
                    font = parent.getFont();
                }
            }
            return font;
        }
        
        @Override
        public Dimension getPreferredSize() {
            Dimension preferredSize = super.getPreferredSize();
            if (DefaultTreeCellEditor.this.renderer != null && DefaultTreeCellEditor.this.getFont() == null) {
                preferredSize.height = DefaultTreeCellEditor.this.renderer.getPreferredSize().height;
                return preferredSize;
            }
            return preferredSize;
        }
    }
    
    public class EditorContainer extends Container
    {
        public EditorContainer() {
            this.setLayout(null);
        }
        
        public void EditorContainer() {
            this.setLayout(null);
        }
        
        @Override
        public void paint(Graphics g) {
            int width = this.getWidth();
            int height = this.getHeight();
            if (DefaultTreeCellEditor.this.editingIcon != null) {
                int calculateIconY = this.calculateIconY(DefaultTreeCellEditor.this.editingIcon);
                if (this.getComponentOrientation().isLeftToRight()) {
                    DefaultTreeCellEditor.this.editingIcon.paintIcon(this, g, 0, calculateIconY);
                }
                else {
                    DefaultTreeCellEditor.this.editingIcon.paintIcon(this, g, width - DefaultTreeCellEditor.this.editingIcon.getIconWidth(), calculateIconY);
                }
            }
            Color borderSelectionColor = DefaultTreeCellEditor.this.getBorderSelectionColor();
            if (borderSelectionColor != null) {
                g.setColor(borderSelectionColor);
                g.drawRect(0, 0, width - 1, height - 1);
            }
            super.paint(g);
        }
        
        @Override
        public void doLayout() {
            if (DefaultTreeCellEditor.this.editingComponent != null) {
                int width = this.getWidth();
                int height = this.getHeight();
                if (this.getComponentOrientation().isLeftToRight()) {
                    DefaultTreeCellEditor.this.editingComponent.setBounds(DefaultTreeCellEditor.this.offset, 0, width - DefaultTreeCellEditor.this.offset, height);
                }
                else {
                    DefaultTreeCellEditor.this.editingComponent.setBounds(0, 0, width - DefaultTreeCellEditor.this.offset, height);
                }
            }
        }
        
        private int calculateIconY(Icon icon) {
            int iconHeight = icon.getIconHeight();
            int height = DefaultTreeCellEditor.this.editingComponent.getFontMetrics(DefaultTreeCellEditor.this.editingComponent.getFont()).getHeight();
            int b = iconHeight / 2 - height / 2;
            int min = Math.min(0, b);
            return this.getHeight() / 2 - (min + (Math.max(iconHeight, b + height) - min) / 2);
        }
        
        @Override
        public Dimension getPreferredSize() {
            if (DefaultTreeCellEditor.this.editingComponent != null) {
                Dimension preferredSize = DefaultTreeCellEditor.this.editingComponent.getPreferredSize();
                preferredSize.width += DefaultTreeCellEditor.this.offset + 5;
                Dimension dimension = (DefaultTreeCellEditor.this.renderer != null) ? DefaultTreeCellEditor.this.renderer.getPreferredSize() : null;
                if (dimension != null) {
                    preferredSize.height = Math.max(preferredSize.height, dimension.height);
                }
                if (DefaultTreeCellEditor.this.editingIcon != null) {
                    preferredSize.height = Math.max(preferredSize.height, DefaultTreeCellEditor.this.editingIcon.getIconHeight());
                }
                preferredSize.width = Math.max(preferredSize.width, 100);
                return preferredSize;
            }
            else {
                return new Dimension(0, 0);
            }
        }
    }
}
